from enum import Enum


class TrajectoryExcel(Enum):
    name_ = "trajectory"
    start = "start"
    end = "end"
    label_property = "label_property"
    label_value = "label_value"
    step = "step"
