from enum import Enum


class ExcelSheet(Enum):
    help = "Help"
    help_extractors = "HelpExtractors"
    help_reducers = "HelpReducers"
    help_autoclusters = "HelpAutoclusters"
    help_digesters = "HelpDigesters"
    settings = "Settings"
    files = "Files"
    bands = "Bands"
    integrations = "Integrations"
    ranges = "Ranges"
    extractors = "Extractors"
    reducers = "Reducers"
    autoclusters = "Autoclusters"
    digesters = "Digesters"
    trajectories = "Trajectories"
